	DEVICE ZXSPECTRUM48

	ORG 33000
Start
	
	;create frames of animation
	INCLUDE <Frames.asm>
	
Start1	
	LD A,1
	LD (Level),A

;draw game screen
Start2	
	DI
	LD SP,65000
	EI

	CALL NewLevelLogo
	CALL ClearScreen
	
;find current level colors
	LD A,(Level)	
	LD DE,Level1Colors-4
Start3
	INC DE	
	INC DE	
	INC DE	
	INC DE	
	DEC A
	JR NZ,Start3
	
	CALL DrawLocation
	
;set initial data
	LD HL, Variables
	
;set initial positions of truck and mines	
	LD HL,Variables	
	LD DE,InitPositions
	LD B,7				;do truck and 6 mines
	
SetLevel1
	LD A,(DE)
	LD (HL),A
	INC HL				;write initial Y pos for object
	INC DE
	
	LD A,(DE)
	LD (HL),A
	INC HL				;write initial X pos
	INC DE
	
	XOR A
	LD (HL),A			;HL position write to HL=0
	INC HL				;so when clearing XOR routine runs for the first time
	LD (HL),A			;it will draw into ROM and won't change anything on the screen
	INC HL				;later the HLs will be calculated and clearing routine will be running
	LD (HL),A			;over already drawn sprites, erasing them
	
	INC HL				;going to next object
	DJNZ SetLevel1
	
	LD (HL),A			;set distance=0

	
	
MainLoop
	HALT
	CALL MoveBackgroundLeft
	
	CALL DrawMinesXor		;clear mines
	CALL MoveMines
	CALL DrawMinesXor		;draw mines at new positions
	
	CALL DrawTruckXor		;clear truck at old position
	CALL MoveTruck			
	CALL DrawTruckXor		;draw truck at new position

	LD HL,Timer				;change game time
	LD A,(HL)
	INC A
	AND 00001111b
	LD (HL),A
	OR A
	CALL Z,ReduceDistance
	
	CALL CheckCollision
	
	JR MainLoop	
	

;-------------
ReduceDistance
	DEC HL		;go to distance	
	INC (HL)	;increase covered distance
	
	LD A,(HL)	;end of level?
	CP 192
	JR Z,GoToNewLevel
	
;decrease distance bar
	LD HL,16384+100+23
	LD BC,24*256+5				;width+nr of pixel lines
	
ReduceDistance1	
	OR A						;carry=0
	PUSH BC
	PUSH HL
	
ReduceDistance2
	RL (HL)						;scroll bar left						
	DEC HL
	DJNZ ReduceDistance2
	
	POP HL
	POP BC
	CALL OnePixelLineDown		;do next pixel line
	DEC C
	JR NZ,ReduceDistance1	
	RET
	
GoToNewLevel
	CALL FinalSound
	LD HL,Level	
	INC (HL)					;next level nr
	LD A,(HL)
	CP 5
	JR Z,GameWin				;end of game
	
	JP Start2

NewLevelLogo
	CALL ClearScreen
	LD HL,18432+143
	LD DE,Go
	LD BC,2*256+6
	CALL DrawXor
	CALL Wait
	RET
	
GameWin	
	CALL ClearScreen
	LD HL,18432+142
	LD DE,Sign
	LD BC,4*256+16
	CALL DrawXor
	CALL Wait				;display game logo after finishing the game
							;go to game over after it
GameOver	
	CALL ClearScreen
	LD HL,18432+143
	LD DE,End
	LD BC,2*256+5
	CALL DrawXor
	CALL Wait
	JP Start1
	
FinalSound
	LD B,40
	LD HL, 100				;initial frequency
	
FinalSound2
	PUSH BC
	PUSH HL
	LD DE,10					;length
	CALL 949
	POP HL
	LD DE,20
	ADD HL,DE
	
	POP BC
	DJNZ FinalSound2
	RET

	
Wait
	LD B,130
Wait2
	HALT
	DJNZ Wait2
	RET	
		
	
;-------------
;initial positions
InitPositions
	.BYTE 160,8		;truck Y,truck X
	.BYTE 134,45	;mine 1 Y,mine1 X
	.BYTE 172,80
	.BYTE 185,120
	.BYTE 170,165
	.BYTE 140,210
	.BYTE 155,248	;mine 6
;-------------
Variables	.EQU 34500

TruckPosXY	.EQU 34500
TruckY		.EQU 34500
TruckX		.EQU 34501
TruckPosHL	.EQU 34502
TruckShift	.EQU 34504

MineTable	.EQU 34505

Mine1Y		.EQU 34505
Mine1X		.EQU 34506
Mine1HL		.EQU 34507
Mine1Shift	.EQU 34509

;for all the mines the data structure is the same
;Mine2
;Mine3
;Mine4
;Mine5
;Mine6

Distance	.EQU 34535
Timer		.EQU 34536
Level		.EQU 34537




	INCLUDE <Truck.asm>
	INCLUDE <Mines.asm>
	INCLUDE <Graphics.asm>
	
	SAVESNA "BSands.sna",Start
	SAVEBIN "Bsands.bin",33000,1120
